/* _NVRM_COPYRIGHT_BEGIN_
 *
 * Copyright 2001 by NVIDIA Corporation.  All rights reserved.  All
 * information contained herein is proprietary and confidential to NVIDIA
 * Corporation.  Any use, reproduction, or disclosure without the written
 * permission of NVIDIA Corporation is prohibited.
 *
 * _NVRM_COPYRIGHT_END_
 */


#ifndef _NV_LINUX_H_
#define _NV_LINUX_H_

#include "nv.h"

#include <linux/config.h>

#if defined (CONFIG_MODVERSIONS) && !defined (MODVERSIONS)
#  define MODVERSIONS
#endif

#if defined (MODVERSIONS)
#include <linux/modversions.h>
#endif

#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/version.h>

#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 2, 12)
#  error This driver does not support 2.2.11 or earlier kernels!
#elif LINUX_VERSION_CODE < KERNEL_VERSION(2, 3, 0)
#  define KERNEL_2_2
#elif LINUX_VERSION_CODE < KERNEL_VERSION(2, 4, 0)
#  error This driver does not support 2.3.x development kernels!
#elif LINUX_VERSION_CODE < KERNEL_VERSION(2, 5, 0)
#  define KERNEL_2_4
#elif LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 0)
#  error This driver does not support 2.5.x development kernels!
#  define KERNEL_2_5
#else
#  error This driver does not support 2.6.x or newer kernels!
#endif

#if defined (CONFIG_SMP) && !defined (__SMP__)
#define __SMP__
#endif

#include <linux/types.h>            /* pic_t, size_t, __u32, etc        */
#include <linux/errno.h>            /* error codes                      */
#include <linux/stddef.h>           /* NULL, offsetof                   */
#include <linux/wait.h>             /* wait queues                      */
#include <linux/tqueue.h>           /* struct tq_struct                 */

#include <linux/slab.h>             /* kmalloc, kfree, etc              */
#include <linux/vmalloc.h>          /* vmalloc, vfree, etc              */

#include <linux/poll.h>             /* poll_wait                        */
#include <linux/delay.h>            /* mdelay, udelay                   */

#include <linux/pci.h>              /* pci_find_class, etc              */
#include <linux/wrapper.h>          /* mem_map_reserve                  */
#include <linux/interrupt.h>        /* mark_bh, init_bh, remove_bh      */

#include <asm/system.h>             /* cli, sli, save_flags             */
#include <asm/io.h>                 /* ioremap, virt_to_phys            */
#include <asm/uaccess.h>            /* access_ok                        */
#include <asm/page.h>               /* PAGE_OFFSET                      */
#include <asm/pgtable.h>            /* pte bit definitions              */

#if !defined (KERNEL_2_2)
#include <linux/spinlock.h>
#include <asm/semaphore.h>
#include <linux/highmem.h>
#else
#include <asm/spinlock.h>
#include <asm/semaphore.h>
#endif

#ifdef CONFIG_PROC_FS
#include <linux/proc_fs.h>
#endif

#ifdef CONFIG_DEVFS_FS
#include <linux/devfs_fs_kernel.h>
#endif

#ifdef CONFIG_KMOD
#include <linux/kmod.h>
#endif

#ifdef CONFIG_PM
#include <linux/pm.h>
#endif

#ifdef CONFIG_MTRR
#include <asm/mtrr.h>
#endif

#ifdef CONFIG_KDB
#include <asm/kdb.h>
#endif

#if defined (NVCPU_IA64_LINUX)
#  if LINUX_VERSION_CODE < KERNEL_VERSION(2, 4, 13)
#    error This driver does not support 2.4.12 or earlier kernels!
#  endif
#endif

#include <linux/autoconf.h>

#if defined (CONFIG_AGP) || defined (CONFIG_AGP_MODULE)
#ifndef NOAGPGART
#  define AGPGART
#  include <linux/agp_backend.h>
#  include <linux/agpgart.h>
#endif
#endif

#if !defined (KERNEL_2_2)
#  define LINUX_VMA_OFFS(vma)           (((vma)->vm_pgoff) << PAGE_SHIFT)
#  define GET_MODULE_SYMBOL(mod,sym)	(const void *) inter_module_get(sym)
#  define PUT_MODULE_SYMBOL(sym)        inter_module_put((char *) sym)
#  define GET_MAP_NR(phys_page)         virt_to_page(__va(phys_page))
#  define MEM_MAP_READ_COUNT(map_nr)    (atomic_read(&(map_nr)->count))
#  define MEM_MAP_INC_COUNT(map_nr)     (atomic_inc(&(map_nr)->count))
#  define MEM_MAP_DEC_COUNT(map_nr)     (atomic_dec(&(map_nr)->count))
#  define GET_EVENT_QUEUE(nv)           ((struct __wait_queue_head *) ((nv)->event_queue))
#else
#  define in_irq()                      (local_irq_count[smp_processor_id()])
#  define LINUX_VMA_OFFS(vma)           ((vma)->vm_offset)
#  define GET_MODULE_SYMBOL(mod, sym)   (void*) get_module_symbol((mod), (sym))
#  define PUT_MODULE_SYMBOL(sym)
#  define GET_MAP_NR(phys_page)         MAP_NR(__va(phys_page))
#  define MEM_MAP_READ_COUNT(map_nr)    (atomic_read(&mem_map[map_nr].count))
#  define MEM_MAP_INC_COUNT(map_nr)     (atomic_inc(&mem_map[map_nr].count))
#  define MEM_MAP_DEC_COUNT(map_nr)     (atomic_dec(&mem_map[map_nr].count))
#  define GET_EVENT_QUEUE(nv)           ((struct wait_queue **) &((nv)->event_queue))
#endif

#define NV_PAGE_ALIGN(addr)             ( ((addr) + PAGE_SIZE - 1) / PAGE_SIZE)
#define NV_MASK_OFFSET(addr)            ( (addr) & (PAGE_SIZE - 1) )

#ifndef NV01_ROOT
#define NV01_ROOT 0x00000000
#endif 

#if defined (DEBUG)
#define NV_DEBUG_LOCKS 1
#endif

#if defined(NVCPU_IA64)
#define NV_GFP_HW (GFP_KERNEL | __GFP_DMA)
#else
#define NV_GFP_HW (GFP_KERNEL)
#endif
 
#define calc_order(size, order)                         \
    {                                                   \
        order = 0;                                      \
        while ( ((1 << order) * PAGE_SIZE) < (size))    \
        {                                               \
            order++;                                    \
        }                                               \
    }

/*
 * An allocated bit of memory using NV_MEMORY_ALLOCATION_OFFSET
 *   looks like this in the driver
 */

typedef struct nv_alloc_s {
    struct nv_alloc_s *next;    
    struct vm_area_struct *vma;
    unsigned int   process_id;
    unsigned int   thread_gid;
    unsigned int   num_pages;
    void          *direct_kmapping;
    void         **page_table;          /* list of physical pages allocated */
    void          *agp_mapping;         /* ptr to linear agp mapping of pages */
    /* for nvagp */
    unsigned int   class;
    void          *priv_data;
} nv_alloc_t;


/* linux-specific version of old nv_state_t */
/* this is a general os-specific state structure. the first element *must* be
   the general state structure, for the generic unix-based code */
typedef struct {
    nv_state_t nv_state;

    nv_alloc_t *alloc_queue;

    // bottom half interrupt handler info; per device
    struct tq_struct *bh;

    U032 vblank_notifier;
    U032 waiting_for_vblank;

    /* queue for for NV's OS events */
    void *event_queue;

    /* per-device locking mechanism for access to core rm */
    spinlock_t rm_lock;
    unsigned long rm_pflags;

    /* lock for linux-specific data, not used by core rm */
    /* XXX eventually change to sleeping locks? */
    spinlock_t ldata_lock;
    unsigned long ldata_pflags;

    /* lock for linux-specific alloc queue */
    /* XXX eventually change to sleeping locks? */
    spinlock_t at_lock;
    unsigned long at_pflags;

    /* lock for bottom half, not used by core rm */
    spinlock_t bh_lock;
    unsigned long bh_pflags;

#if defined(NV_DEBUG_LOCKS)
    /* only used in debug builds to track who has the locks */
    unsigned long rm_locked;
    unsigned long rm_waiting;
    unsigned long ldata_locked;
    unsigned long ldata_waiting;
    unsigned long at_locked;
    unsigned long at_waiting;
    unsigned long bh_locked;
#endif

} nv_linux_state_t;


/*
 * poll(2) support for event notifiers
 *
 * A notifer can be turned into an "OS" event.
 * On UNIX, this means you can block using poll() while waiting for any
 * of your OS events to fire.
 *
 * When poll() on the NV file descriptor notices that you have one or more
 * events, it returns POLLPRI.  At that time it clears the count of fired
 * notifiers.
 *
 * Upon return from poll() just poll your notifiers.
 */

/*
 * Per-file private data structure.
 * We keep stuff here that is separate on a per-file basis.
 */

typedef struct {
    U032 any_fired_notifiers;
    void *nv_ptr;
} nv_file_private_t;

/* for the card devices */
#define NV_GET_NVL_FROM_FILEP(filep) \
    (nv_linux_state_t*) (((nv_file_private_t *)((filep)->private_data))->nv_ptr)

/* for the control device */
#define NV_GET_NV_FROM_FILEP(filep) \
    (nv_state_t*) (((nv_file_private_t *)((filep)->private_data))->nv_ptr)

#define NV_HIDE_IN_FILEP(filep, nvp) \
    (((nv_file_private_t *)((filep)->private_data))->nv_ptr = (void *) (nvp))
    

#define NV_GET_NVL_FROM_NV_STATE(nv) \
    ((nv_linux_state_t *) nv->os_state)

#define NV_STATE_PTR(nvl)   (&((nvl)->nv_state))

/* all of the below kernel calls could sleep.
 * if we sleep with the nv lock, we could hang an incoming interrupt
 * and lock the system. So, drop the lock, perform the call, and
 * regrab the lock.
 */

#define NV_VMALLOC(ptr, size) \
    { \
        (void *) ptr = vmalloc(size); \
    }

#define NV_VFREE(ptr) \
    { \
        vfree((void *) ptr); \
    }

/* only use this because GFP_KERNEL may sleep..
 * GFP_ATOMIC is ok, it won't sleep
 */
#define NV_KMALLOC(ptr, size) \
    { \
        (void *) ptr = kmalloc(size, GFP_KERNEL); \
    }

#define NV_KFREE(ptr) \
    { \
        kfree((void *) ptr); \
    }

#endif  /* _NV_LINUX_H_ */
