/* _NVRM_COPYRIGHT_BEGIN_
 *
 * Copyright 1999-2001 by NVIDIA Corporation.  All rights reserved.  All
 * information contained herein is proprietary and confidential to NVIDIA
 * Corporation.  Any use, reproduction, or disclosure without the written
 * permission of NVIDIA Corporation is prohibited.
 *
 * _NVRM_COPYRIGHT_END_
 */


#ifndef _NV_H_
#define _NV_H_

#include <nvtypes.h>

/* Nvidia's reserved major device number
 * Set this to 0 to request dynamic major number allocation. */
#define NV_MAJOR_DEVICE_NUMBER 195

/* most cards in a single system */
#define NV_MAX_DEVICES 8

typedef struct nv_ioctl_memory_vtop
{
    NvP64 buffer;
    NvU32 byte_length;
    NvP64 physical_addresses;
} nv_ioctl_memory_vtop_t;

#define NV_IOCTL_MAGIC      'F'
#define NV_IOCTL_MEMORY_VTOP    _IOWR(NV_IOCTL_MAGIC, 1, nv_ioctl_memory_vtop_t)

typedef struct nv_ioctl_card_info
{
    NvU32    flags;               /* see below                   */
    NvU32    bus;		  /* bus number (PCI, AGP, etc)  */
    NvU32    slot;                /* card slot                   */
    NvU32    vendor_id;           /* PCI vendor id               */
    NvU32    device_id;
    NvU32    interrupt_line;
    NvU32    reg_address;         /* register aperture           */
    NvU32    reg_size;
    NvU32    fb_address;          /* framebuffer aperture        */
    NvU32    fb_size;
} nv_ioctl_card_info_t;

#define NV_IOCTL_CARD_INFO_FLAG_PRESENT       0x0001

#define NV_IOCTL_CARD_INFO     \
    _IOWR(NV_IOCTL_MAGIC, 2, NvU8[NV_MAX_DEVICES*sizeof(nv_ioctl_card_info_t)])


/* allow interrupts to be turned off/on.
 * hopefully just for debug or working around buggy Xservers :-)
 * Specify 'arg' as a 0 (disable) or non-zero (enable)
 */
#define NV_IOCTL_INTERRUPTS     _IOWR(NV_IOCTL_MAGIC, 3, sizeof(void *))

typedef struct nv_ioctl_primary_card
{
    NvU32 index;			/* primary card index */
} nv_ioctl_primary_card_t;

/* notify resman which card is primary */
#define NV_IOCTL_PRIMARY_CARD  _IOWR(NV_IOCTL_MAGIC, 4, nv_ioctl_primary_card_t)

/* debug tool to force module count to 0 */
#define NV_IOCTL_MODULE_RESET  _IO(NV_IOCTL_MAGIC, 99)


/* mmap(2) offsets */

#define NV_MMAP_REG_OFFSET              (0)
#define NV_MMAP_FB_OFFSET                (256 * 1024 * 1024)
/* Magic mmap() offset for the memory allocator */
#define NV_MMAP_ALLOCATION_OFFSET (1 * 1024 * 1024 * 1024)
/* AGP indicator, intended to be or'd with magic allocator offset */
#define NV_MMAP_AGP_OFFSET ((unsigned long)(2)*(1024 * 1024 * 1024))

/* duplicated from nvos.h for external builds */
#ifndef NVOS_AGP_CONFIG_DISABLE_AGP
#  define NVOS_AGP_CONFIG_DISABLE_AGP (0x00000000)
#endif
#ifndef NVOS_AGP_CONFIG_NVAGP
#  define NVOS_AGP_CONFIG_NVAGP       (0x00000001)
#endif
#ifndef NVOS_AGP_CONFIG_OSAGP
#  define NVOS_AGP_CONFIG_OSAGP       (0x00000002)
#endif
#ifndef NVOS_AGP_CONFIG_ANYAGP
#  define NVOS_AGP_CONFIG_ANYAGP      (0x00000003)
#endif

#ifdef __KERNEL__

/*
 * ptr arithmetic convenience
 */

typedef union
{
    volatile V008 Reg008[1];
    volatile V016 Reg016[1];
    volatile V032 Reg032[1];
} nv_hwreg_t, * nv_phwreg_t;

/*
 * data structure to keep track of open clients for abnormal termination
 * cleanup.  See nv.c for discussion of cleanup issues
 * NOTE: this can't be kept in 'nv' since the nv current during root allocate
 *       is the control 'nv'.
 */

#define NV_MAX_CLIENTS 128         /* *system-wide* limit, not per device */
typedef struct {
    NvU32  client;
    int    pid;
    void  *file;
} nv_client_t;
extern nv_client_t nv_clients[NV_MAX_CLIENTS];


#define SAVED_BIOS_IMAGE_SIZE 0x10000
#define NUM_FB_PAGES_CACHED 10

typedef struct {
    NvU32 address;
    NvU32 size;
    NvU32 *map;
    nv_phwreg_t map_u;
} nv_aperture_t;

typedef struct
{
    NvU32 fp_state_00;
    NvU32 fp_state_01;
    NvU32 fp_state_02;
    NvU32 fp_state_03;
    NvU32 fp_state_04;
    NvU32 fp_state_05;
    NvU32 fp_state_06;
    NvU32 fp_state_07;
    NvU32 fp_state_08;
    NvU32 fp_state_09;
    NvU32 fp_state_10;
    NvU32 fp_state_11;
    NvU32 fp_state_12;
    NvU32 fp_state_13;
    NvU32 fp_state_14;
    NvU32 fp_state_15;
    NvU32 fp_state_16;
    NvU32 fp_state_17;
    NvU32 fp_state_18;
    NvU32 fp_state_19;
    NvU32 fp_state_20;
    NvU32 fp_state_21;
    NvU32 fp_state_22;
    NvU32 fp_state_23;
    NvU32 fp_state_24;
} nv_fpstate_t;

/*
 * per device state
 */

typedef struct
{
    void  *pdev;		    /* you know who */
    void  *os_state;                /* os-specific version of this file */

    int    flags;
    int    device_number;
    int    usage_count;

    /* mapped addresses */
    NvU32  *rom;

    /* PCI config info */
    int bus;
    int slot;
    int vendor_id;
    int device_id;

    /* physical characteristics */
    nv_aperture_t regs;
    nv_aperture_t fb;

    /* stored flat panel state */
    nv_fpstate_t fp;

    NvU32  interrupt_line;

    U032 agp_config;
    U032 agp_buffers;
    U032 agp_teardown;

    /* keep track of any pending bottom-halves */
    int bh_count;

    /* copy of the video bios in system memory */
    /* used by general resman code to query bios-set values */
    void *vbios;

    nv_aperture_t fb_cache[NUM_FB_PAGES_CACHED];

} nv_state_t;

/*
 * flags
 */

#define NV_FLAG_OPEN            0x0001
#define NV_FLAG_POSTED          0x0002
#define NV_FLAG_WAITING         0x0004
#define NV_FLAG_CONTROL         0x0010
#define NV_FLAG_HOTKEY_OCCURRED 0x0100

#define NV_AGP_ENABLED(nv)      ((nv)->agp_config)
#define NV_AGP_DISABLED(nv)     (!(nv)->agp_config)
#define NV_NVAGP_ENABLED(nv)    ((nv)->agp_config & NVOS_AGP_CONFIG_NVAGP)
#define NV_OSAGP_ENABLED(nv)    ((nv)->agp_config & NVOS_AGP_CONFIG_OSAGP)

/*
** where we hide our nv_state_t * in a pdev
** subject to change...
*/
#define NV_HIDE_NV_IN_PDEV(pdev) \
                     ((nv_state_t *) (pdev)->DBmpDeviceExtension)

#define NV_WAITING_FOR_VBLANK(nv) \
                     (nv->flags & NV_FLAG_WAITING)

/*
 * driver internal interfaces
 */

/* need a fake device number for control device; just to flag it for msgs */
#define NV_CONTROL_DEVICE_NUMBER     100


/*
 * ---------------------------------------------------------------------------
 *
 * Function prototypes for UNIX specific OS interface.
 *
 * ---------------------------------------------------------------------------
 */

S032   nv_vtop                      (nv_ioctl_memory_vtop_t *, S032);
VOID*  nv_find_kernel_mapping       (nv_state_t *, VOID *);
ULONG  nv_get_phys_address          (ULONG);

VOID   nv_lock_rm                   (nv_state_t *);
VOID   nv_unlock_rm                 (nv_state_t *);
VOID   nv_lock_bh                   (nv_state_t *);
VOID   nv_unlock_bh                 (nv_state_t *);

VOID   nv_post_event                (VOID *);
VOID   nv_post_vblank               (nv_state_t *);
VOID   nv_set_hotkey_occurred_flag  (VOID);
S032   nv_int10h_call               (nv_state_t *, U032 *, U032 *, U032 *, U032 *, VOID *);

S032   nv_alloc_pages               (nv_state_t *, VOID **, U032, U032, U032, VOID **);
S032   nv_free_pages                (nv_state_t *, VOID **, U032, U032, VOID **);

S032   nv_agp_init                  (nv_state_t *, VOID **, VOID **, VOID *, U032);
S032   nv_agp_teardown              (nv_state_t *);
S032   nv_agp_translate_address     (nv_state_t *, VOID *, U032, U032 *);
S032   nv_agp_retrieve_mapping      (nv_state_t *, VOID *, ULONG *);


/*
 * ---------------------------------------------------------------------------
 *
 * Function prototypes for Resource Manager interface.
 *
 * ---------------------------------------------------------------------------
 */

BOOL       rm_init_rm               (VOID);
BOOL       rm_shutdown_rm           (VOID);
BOOL       rm_init_adapter          (nv_state_t *);
BOOL       rm_disable_adapter       (nv_state_t *);
BOOL       rm_shutdown_adapter      (nv_state_t *);
BOOL       rm_ioctl                 (nv_state_t *, VOID *, U032, VOID *);
BOOL       rm_isr                   (U032, U032 *);
VOID       rm_isr_bh                (VOID *);
RM_STATUS  rm_power_management      (nv_state_t *, U032, U032);
U032       rm_get_vbios_version     (nv_state_t *, U032 *, U032 *, U032 *, U032 *, U032 *);
VOID       rm_free_unused_clients   (nv_state_t *, U032, VOID *);

VOID       rm_update_agp_config     (nv_state_t *);
BOOL       rm_init_agp              (nv_state_t *);
BOOL       rm_shutdown_agp          (nv_state_t *);

RM_STATUS  rm_alloc_agp_pages       (nv_state_t *, VOID **, U032, U032, VOID **);
RM_STATUS  rm_map_agp_pages         (nv_state_t *, VOID **, U032, VOID *);
RM_STATUS  rm_free_agp_pages        (nv_state_t *, VOID **, VOID *);

NvUI64     nv_rdtsc                 (VOID);


/*
 * ---------------------------------------------------------------------------
 *
 * Function prototypes for error/message reporting.
 *
 * ---------------------------------------------------------------------------
 */

#define NV_ERROR_ERRNO      (1<<31)     /* hi bit; use 'errno' */
#define NV_ERROR_PANIC      (1<<30)     /* err is fatal; panic */
#define NV_ERROR_MESSAGE    (1<<29)     /* just msg; not error */
#define NV_ERROR_STATUS     (1<<28)     /* rtems status in error */
#define NV_ERROR_WARNING    (1<<27)     /* just warning; no error */

#define NV_ERROR_MASK   \
(                       \
    NV_ERROR_ERRNO   |  \
    NV_ERROR_PANIC   |  \
    NV_ERROR_MESSAGE |  \
    NV_ERROR_WARNING |  \
    NV_ERROR_STATUS     \
)

int  nv_error             (int, const char *, ...);
void nv_panic             (const char *, ...) __attribute__ ((noreturn));
void nv_abort             (const char *, ...) __attribute__ ((noreturn));
void nv_message           (const char *, ...);
void nv_warning           (const char *, ...);
void nv_error_new_failed  (void);
int  nv_printf            (const char *, ...);
int  nv_debug             (int level, const char *, ...);

#endif /* __KERNEL__ */

#endif
